{ This demo program shows how to use the statistics functions implemented
  in ChroMathsome - Statistics.

  The data points must be entered in a statistical vector (StatVector, see help file).
  The general scheme is as follows:
  - How many data points? -> x;
  - Initialize a StatVector V to hold just x values -> InitStatVector(V, x);
  - Do something useful with V;
  - Clear memory consumed by V -> DestroyStatVector(V).
  Although this may look cumbersome at first sight, study the examples below
  and you'll see how easily this scheme can be applied.
  Note that you must reference individual elements of this statistical vector as
  with a pointer variable (using a caret ^ ).

  I've added (very) simple graphing routines for the linear/polynomial regression functions as an example.
  This draws both the data points and the found curve.
  This way you can get an idea of how well the resulting curve fits the data points.
  WARNING: do NOT change anything in the code below or in the file SGraph.pas unless you are
  certain of what you are doing.
  Both units interact with each other in the following way (example from straight line regression):

    SGraph uses two StatVectors nl. SX and SY.
    Both StatVectors are initialized in this unit (Stat1) as:

    InitStatVector(SGraph.SX, 6);  (* 6 is number of data points *)
    InitStatVector(SGraph.SY, 6);

    Next, we copy the values of the original StatVectors to SX and SY:

    for i:= 1 to V1.Dimension do
    begin
    SGraph.SX.Vector^[i]:= V1.Vector^[i];
    SGraph.SY.Vector^[i]:= V2.Vector^[i];
    end;

    Then we initialize some variables of SGraph and show the StatGraphForm (SGraph)
    with both the data points and the curve drawn on it:

    SGraph.DrawLinRegCurve:= true;  (* if false -> draw polynom curve *)
    SGraph.CurType:= Linear;
    SGraph.ca:= a;
    SGraph.cb:= b;
    StatGraphForm.ShowModal;

    When StatGraphForm closes, it automatically destroys SX and SY.
    So, do not destroy SX and/or SY in this unit (Stat1.pas)!

    Remember, the graphing routines are only added as an example.
    Don't expect them to create a graph with high artistic value...
    The bells and whistles you have to add yourself.

  The reader may do with this code whatever he/she wants.
  Author: John Merrit (john.merrit@net4all.be)
  Release date: October 1 '97

  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  Brought to you by the letter "M".             }

unit Stat1;

interface

uses
  WinTypes, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls;

type

  TForm1 = class(TForm)
    Memo1: TMemo;
    MainMenu1: TMainMenu;
    Exit1: TMenuItem;
    Basic1: TMenuItem;
    Regression1: TMenuItem;
    PolyRegression1: TMenuItem;
    TestStat1: TMenuItem;
    Pairedt1: TMenuItem;
    chi1: TMenuItem;
    Sum1: TMenuItem;
    Mean1: TMenuItem;
    StdDeviation1: TMenuItem;
    N1: TMenuItem;
    Firstsecondthirdandfourthmoment1: TMenuItem;
    Coeffofskewness1: TMenuItem;
    Coeffofkurtosis1: TMenuItem;
    Linear1: TMenuItem;
    Logarithmic1: TMenuItem;
    Exponential1: TMenuItem;
    Power1: TMenuItem;
    N2: TMenuItem;
    Bestfit1: TMenuItem;
    N3: TMenuItem;
    Multlinregression1: TMenuItem;
    Twoindepvar1: TMenuItem;
    Threeindepvariables1: TMenuItem;
    Parabolic1: TMenuItem;
    Cubic1: TMenuItem;
    Version1: TMenuItem;
    N4: TMenuItem;
    Close1: TMenuItem;
    Higher1: TMenuItem;
    N5: TMenuItem;
    BestFit2: TMenuItem;
    Variance1: TMenuItem;
    PopVariance1: TMenuItem;
    N6: TMenuItem;
    Covariance1: TMenuItem;
    PopCovariance1: TMenuItem;
    tStat1: TMenuItem;
    Example1: TMenuItem;
    procedure Sum1Click(Sender: TObject);
    procedure Mean1Click(Sender: TObject);
    procedure Moments(Sender: TObject);
    procedure Linear1Click(Sender: TObject);
    procedure Logarithmic1Click(Sender: TObject);
    procedure Exponential1Click(Sender: TObject);
    procedure Power1Click(Sender: TObject);
    procedure Bestfit1Click(Sender: TObject);
    procedure Parabolic1Click(Sender: TObject);
    procedure Cubic1Click(Sender: TObject);
    procedure Twoindepvar1Click(Sender: TObject);
    procedure Threeindepvariables1Click(Sender: TObject);
    procedure Pairedt1Click(Sender: TObject);
    procedure chi1Click(Sender: TObject);
    procedure Version1Click(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure Higher1Click(Sender: TObject);
    procedure BestFit2Click(Sender: TObject);
    procedure tStat1Click(Sender: TObject);
    procedure Example1Click(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses Statistc, SGraph;

procedure TForm1.Sum1Click(Sender: TObject);
var x: extended;
    V: StatVector;
begin
    InitStatVector(V, 5);
    try
    V.Vector^[1]:= 5;
    V.Vector^[2]:= 2;
    V.Vector^[3]:= 4;
    V.Vector^[4]:= 3;
    V.Vector^[5]:= 7;
    x:= Sum(V);
    Memo1.Lines.add('Sum: ' + floattostr(x));
    finally
    DestroyStatVector(V);
    end;
end;

procedure TForm1.Mean1Click(Sender: TObject);
var DMean, Deviation: extended;
    V, V2: StatVector;
begin
    InitStatVector(V, 7);
    InitStatVector(V2, 7);
    try
    V.Vector^[1]:= 26;
    V.Vector^[2]:= 30;
    V.Vector^[3]:= 44;
    V.Vector^[4]:= 50;
    V.Vector^[5]:= 62;
    V.Vector^[6]:= 68;
    V.Vector^[7]:= 74;

    V2.Vector^[1]:= 92;
    V2.Vector^[2]:= 85;
    V2.Vector^[3]:= 78;
    V2.Vector^[4]:= 81;
    V2.Vector^[5]:= 54;
    V2.Vector^[6]:= 51;
    V2.Vector^[7]:= 40;

    if Sender = Mean1 then
    begin
    DMean:= Mean(V);
    Memo1.lines.add('Mean: ' + FloatTostr(DMean))
    end
    else if Sender = StdDeviation1 then
    begin
    Deviation:= StdDev(V);
    Memo1.lines.add('Standard deviation: ' + FloatTostr(Deviation));
    end
    else if Sender = Variance1 then
    begin
    Deviation:= Variance(V);
    Memo1.lines.add('Variance: ' + FloatTostr(Deviation));
    end
    else if Sender = PopVariance1 then
    begin
    Deviation:= PopnVariance(V);
    Memo1.lines.add('Population variance: ' + FloatTostr(Deviation));
    end
    else if Sender = CoVariance1 then
    begin
    Deviation:= CoVariance(V, V2);
    Memo1.lines.add('Covariance: ' + FloatTostr(Deviation));
    end
    else if Sender = PopCoVariance1 then
    begin
    Deviation:= PopnCoVariance(V, V2);
    Memo1.lines.add('Population covariance: ' + FloatTostr(Deviation));
    end;
    finally
    DestroyStatVector(V);
    DestroyStatVector(V2);
    end;
end;

{ Calculate the 1st, 2nd, 3rth and 4th moments and the cofficients of skewness and
  kurtosis for the data in Data.  }
procedure TForm1.Moments(Sender: TObject);
var M1, M2, M3, M4, Skew, Kurtosis: extended;
    V: StatVector;
begin
    InitStatVector(V, 9);
    try
    V.Vector^[1]:= 2.1;
    V.Vector^[2]:= 3.5;
    V.Vector^[3]:= 4.2;
    V.Vector^[4]:= 6.5;
    V.Vector^[5]:= 4.1;
    V.Vector^[6]:= 3.6;
    V.Vector^[7]:= 5.3;
    V.Vector^[8]:= 3.7;
    V.Vector^[9]:= 4.9;
    MomentSkewKurtosis(V, M1, M2, M3, M4, Skew, Kurtosis);
    if Sender = Firstsecondthirdandfourthmoment1 then
    Memo1.lines.add('Moments: ' + FloatTostr(M1) + ' ' + FloatTostr(M2) + ' ' + FloatTostr(M3) +
    ' ' + FloatTostr(M4))
    else if Sender = CoeffofSkewness1 then
    Memo1.lines.add('Cofficient of skewness: ' + FloatTostr(skew))
    else if Sender = CoeffofKurtosis1 then
    Memo1.lines.add('Cofficient of kurtosis: ' + FloatTostr(kurtosis));
    finally
    DestroyStatVector(V);
    end;
end;

{ Calculate the coeffic. of a straight line (y = a + b*x) that fits the data supplied in X and Y
  The statistical data must be supplied within two arrays (X and Y).
  Your data is of the form (Xi, Yi).
  Don't forget that the dimensions of X and Y must be the same!!!  }
procedure TForm1.Linear1Click(Sender: TObject);
var a, b, r: extended;
    V1, V2: StatVector;
    i: integer;
begin
    InitStatVector(V1, 6);
    InitStatVector(V2, 6);
    try
    V1.Vector^[1]:= 40.5;
    V1.Vector^[2]:= 38.6;
    V1.Vector^[3]:= 37.9;
    V1.Vector^[4]:= 36.2;
    V1.Vector^[5]:= 35.1;
    V1.Vector^[6]:= 34.6;

    V2.Vector^[1]:= 104.5;
    V2.Vector^[2]:= 102;
    V2.Vector^[3]:= 100;
    V2.Vector^[4]:= 97.5;
    V2.Vector^[5]:= 95.5;
    V2.Vector^[6]:= 94;
    CurveFitting(a,b,r,V1,V2, Linear);
    Memo1.Lines.add('a:= ' + FloatToStr(a));
    Memo1.Lines.add('b:= ' + FloatToStr(b));
    Memo1.Lines.add('r:= ' + FloatToStr(r));

    { Draw the curve and the data points. }
    InitStatVector(SGraph.SX, 6);
    InitStatVector(SGraph.SY, 6);
    for i:= 1 to V1.Dimension do
    begin
    SGraph.SX.Vector^[i]:= V1.Vector^[i];
    SGraph.SY.Vector^[i]:= V2.Vector^[i];
    end;
    SGraph.DrawLinRegCurve:= true;
    SGraph.CurType:= Linear;
    SGraph.ca:= a;
    SGraph.cb:= b;
    StatGraphForm.ShowModal;

    finally
    DestroyStatVector(V1);
    DestroyStatVector(V2);
    end;
end;

{ Same but now for a logarithmic curve (y = a + b*ln(x)).    }
procedure TForm1.Logarithmic1Click(Sender: TObject);
var a, b, r: extended;
    V1, V2, V3: StatVector;
    i: integer;
begin
    InitStatVector(V1, 5);
    InitStatVector(V2, 5);
    InitStatVector(V3, 5);
    try
    V1.Vector^[1]:= 3;
    V1.Vector^[2]:= 4;
    V1.Vector^[3]:= 6;
    V1.Vector^[4]:= 10;
    V1.Vector^[5]:= 12;

    V2.Vector^[1]:= 1.5;
    V2.Vector^[2]:= 9.3;
    V2.Vector^[3]:= 23.4;
    V2.Vector^[4]:= 45.8;
    V2.Vector^[5]:= 60.1;

    CurveFitting(a,b,r,V1,V2, Logarithmic);
    Memo1.Lines.add('a:= ' + FloatToStr(a));
    Memo1.Lines.add('b:= ' + FloatToStr(b));
    Memo1.Lines.add('r:= ' + FloatToStr(r));

    { Find the errors in the regression coefficients of the curve in every point. }
    LinearRegressionDeviations(V3, a, b, V1, V2, Logarithmic);
    Memo1.Lines.add('Deviations of coefficients: ');
    for i:= 1 to V3.Dimension do
    Memo1.Lines.add('Point ' + IntToStr(i) + ': ' + FloatToStr(V3.Vector^[i]));

    { Draw the curve and the data points. }
    InitStatVector(SGraph.SX, 5);
    InitStatVector(SGraph.SY, 5);
    for i:= 1 to V1.Dimension do
    begin
    SGraph.SX.Vector^[i]:= V1.Vector^[i];
    SGraph.SY.Vector^[i]:= V2.Vector^[i];
    end;
    SGraph.DrawLinRegCurve:= true;
    SGraph.CurType:= Logarithmic;
    SGraph.ca:= a;
    SGraph.cb:= b;
    StatGraphForm.ShowModal;

    finally
    DestroyStatVector(V1);
    DestroyStatVector(V2);
    DestroyStatVector(V3);
    end;
end;

{ Same but now for an exponential curve (y = a*e^(b*x)).    }
procedure TForm1.Exponential1Click(Sender: TObject);
var a, b, r: extended;
    V1, V2, V3: StatVector;
    i: integer;
begin
    InitStatVector(V1, 5);
    InitStatVector(V2, 5);
    InitStatVector(V3, 5);
    try
    V1.Vector^[1]:= 0.72;
    V1.Vector^[2]:= 1.31;
    V1.Vector^[3]:= 1.95;
    V1.Vector^[4]:= 2.58;
    V1.Vector^[5]:= 3.14;

    V2.Vector^[1]:= 2.16;
    V2.Vector^[2]:= 1.61;
    V2.Vector^[3]:= 1.16;
    V2.Vector^[4]:= 0.85;
    V2.Vector^[5]:= 0.5;

    CurveFitting(a,b,r,V1,V2, Exponential);
    Memo1.Lines.add('a:= ' + FloatToStr(a));
    Memo1.Lines.add('b:= ' + FloatToStr(b));
    Memo1.Lines.add('r:= ' + FloatToStr(r));

    { Find the errors in the regression coefficients of the curve in every point. }
    LinearRegressionDeviations(V3, a, b, V1, V2, Exponential);
    Memo1.Lines.add('Deviations of coefficients: ');
    for i:= 1 to V3.Dimension do
    Memo1.Lines.add('Point ' + IntToStr(i) + ': ' + FloatToStr(V3.Vector^[i]));

    { Draw the curve and the data points. }
    InitStatVector(SGraph.SX, 5);
    InitStatVector(SGraph.SY, 5);
    for i:= 1 to V1.Dimension do
    begin
    SGraph.SX.Vector^[i]:= V1.Vector^[i];
    SGraph.SY.Vector^[i]:= V2.Vector^[i];
    end;
    SGraph.DrawLinRegCurve:= true;
    SGraph.CurType:= Exponential;
    SGraph.ca:= a;
    SGraph.cb:= b;
    StatGraphForm.ShowModal;

    finally
    DestroyStatVector(V1);
    DestroyStatVector(V2);
    DestroyStatVector(V3);
    end;
end;

{ Same but now for a power curve (y = a*x^b).    }
procedure TForm1.Power1Click(Sender: TObject);
var a, b, r: extended;
    V1, V2: StatVector;
    i: integer;
begin
    InitStatVector(V1, 11);
    InitStatVector(V2, 11);
    try
    V1.Vector^[1]:= 10;
    V1.Vector^[2]:= 12;
    V1.Vector^[3]:= 15;
    V1.Vector^[4]:= 17;
    V1.Vector^[5]:= 20;
    V1.Vector^[6]:= 22;
    V1.Vector^[7]:= 25;
    V1.Vector^[8]:= 27;
    V1.Vector^[9]:= 30;
    V1.Vector^[10]:= 32;
    V1.Vector^[11]:= 35;

    V2.Vector^[1]:= 0.95;
    V2.Vector^[2]:= 1.05;
    V2.Vector^[3]:= 1.25;
    V2.Vector^[4]:= 1.41;
    V2.Vector^[5]:= 1.73;
    V2.Vector^[6]:= 2;
    V2.Vector^[7]:= 2.53;
    V2.Vector^[8]:= 2.98;
    V2.Vector^[9]:= 3.85;
    V2.Vector^[10]:= 4.59;
    V2.Vector^[11]:= 6.02;

    CurveFitting(a,b,r,V1,V2, Power);
    Memo1.Lines.add('a:= ' + FloatToStr(a));
    Memo1.Lines.add('b:= ' + FloatToStr(b));
    Memo1.Lines.add('r:= ' + FloatToStr(r));

    { Draw the curve and the data points. }
    InitStatVector(SGraph.SX, 11);
    InitStatVector(SGraph.SY, 11);
    for i:= 1 to V1.Dimension do
    begin
    SGraph.SX.Vector^[i]:= V1.Vector^[i];
    SGraph.SY.Vector^[i]:= V2.Vector^[i];
    end;
    SGraph.DrawLinRegCurve:= true;
    SGraph.CurType:= Power;
    SGraph.ca:= a;
    SGraph.cb:= b;
    StatGraphForm.ShowModal;

    finally
    DestroyStatVector(V1);
    DestroyStatVector(V2);
    end;
end;

{ Let's see which of the above curves fits the data best.
  The example data points are the same as the for the power curve above.  }
procedure TForm1.Bestfit1Click(Sender: TObject);
var a, b, r: extended;
    CurveType: CurveFittingType;
    V1, V2: StatVector;
begin
    InitStatVector(V1, 11);
    InitStatVector(V2, 11);
    try
    V1.Vector^[1]:= 10;
    V1.Vector^[2]:= 12;
    V1.Vector^[3]:= 15;
    V1.Vector^[4]:= 17;
    V1.Vector^[5]:= 20;
    V1.Vector^[6]:= 22;
    V1.Vector^[7]:= 25;
    V1.Vector^[8]:= 27;
    V1.Vector^[9]:= 30;
    V1.Vector^[10]:= 32;
    V1.Vector^[11]:= 35;

    V2.Vector^[1]:= 0.95;
    V2.Vector^[2]:= 1.05;
    V2.Vector^[3]:= 1.25;
    V2.Vector^[4]:= 1.41;
    V2.Vector^[5]:= 1.73;
    V2.Vector^[6]:= 2;
    V2.Vector^[7]:= 2.53;
    V2.Vector^[8]:= 2.98;
    V2.Vector^[9]:= 3.85;
    V2.Vector^[10]:= 4.59;
    V2.Vector^[11]:= 6.02;

    CurveType:= BestCurveFit(a,b,r,V1,V2);
    case CurveType of
    Linear:begin
           Memo1.Lines.add('Best fit is a straight line y = a + b*x with:');
           Memo1.Lines.add('a:= ' + FloatToStr(a));
           Memo1.Lines.add('b:= ' + FloatToStr(b));
           Memo1.Lines.add('r:= ' + FloatToStr(r));
           end;
    Exponential:begin
                Memo1.Lines.add('Best fit is an exponential curve y = a*exp(b*x) with:');
                Memo1.Lines.add('a:= ' + FloatToStr(a));
                Memo1.Lines.add('b:= ' + FloatToStr(b));
                Memo1.Lines.add('r:= ' + FloatToStr(r));
                end;
    Logarithmic:begin
                Memo1.Lines.add('Best fit is a logarithmic curve y = a + b*ln(x) with:');
                Memo1.Lines.add('a:= ' + FloatToStr(a));
                Memo1.Lines.add('b:= ' + FloatToStr(b));
                Memo1.Lines.add('r:= ' + FloatToStr(r));
                end;
    Power:begin
          Memo1.Lines.add('Best fit is a power curve y = a*x^b with:');
          Memo1.Lines.add('a:= ' + FloatToStr(a));
          Memo1.Lines.add('b:= ' + FloatToStr(b));
          Memo1.Lines.add('r:= ' + FloatToStr(r));
          end;
    end;
    finally
    DestroyStatVector(V1);
    DestroyStatVector(V2);
    end;
end;

{ Fit data in a parabolic curve (y = a + b*x + c*x).
  For polynomial regression:
  Result is returned as a vector (RealVector) that contains the coefficients of the polynom.
  The dimension variable of this vector (automatically adjusted by Statistc.dcu)
  gives the degree of the polynom (polynom degree = vector.dimension - 1).
  Read the result as:
  V.Vector[1] + V.Vector[2]*x + V.Vector[3]*x + ... + V.Vector[n]*x^(n-1) = y.  }
procedure TForm1.Parabolic1Click(Sender: TObject);
var V: realvector;
    V1, V2, V3: StatVector;
    i: integer;
    R: extended;
begin
    InitStatVector(V1, 7);
    InitStatVector(V2, 7);
    InitStatVector(V3, 7);
    try
    V1.Vector^[1]:= 1;
    V1.Vector^[2]:= 2;
    V1.Vector^[3]:= 3;
    V1.Vector^[4]:= 4;
    V1.Vector^[5]:= 5;
    V1.Vector^[6]:= 6;
    V1.Vector^[7]:= 7;

    V2.Vector^[1]:= 5;
    V2.Vector^[2]:= 12;
    V2.Vector^[3]:= 34;
    V2.Vector^[4]:= 50;
    V2.Vector^[5]:= 75;
    V2.Vector^[6]:= 84;
    V2.Vector^[7]:= 128;

    PolynomRegression(V, R, V1, V2, 2);
    Memo1.Lines.add('2nd degree polynom:');
    for i:= 1 to V.Dimension do
    Memo1.Lines.add('coefficient' + IntToStr(i) + ': ' + FloatToStr(V.Vector[i]));
    Memo1.Lines.add('coefficient of det. ' + FloatToStr(r));

    { Find the errors in the regression coefficients of the curve in every point. }
    PolynomRegressionDeviations(V3, V1, V2, V);
    Memo1.Lines.add('Deviations of coefficients:');
    for i:= 1 to V3.Dimension do
    Memo1.Lines.add('Point ' + IntToStr(i) + ': ' + FloatToStr(V3.Vector^[i]));

    { Draw the curve and the data points. }
    InitStatVector(SGraph.SX, 7);
    InitStatVector(SGraph.SY, 7);
    for i:= 1 to V1.Dimension do
    begin
    SGraph.SX.Vector^[i]:= V1.Vector^[i];
    SGraph.SY.Vector^[i]:= V2.Vector^[i];
    end;
    SGraph.Polyn.Dimension:= V.Dimension;
    for i:= 1 to V.Dimension do
    begin
    SGraph.Polyn.Vector[i]:= V.Vector[i];
    end;
    SGraph.DrawLinRegCurve:= false; { polynom }
    StatGraphForm.ShowModal;

    finally
    DestroyStatVector(V1);
    DestroyStatVector(V2);
    DestroyStatVector(V3);
    end;
end;

{ Same but now for a cubic curve (y = a + b*x + c*x + d*x).    }
procedure TForm1.Cubic1Click(Sender: TObject);
var V: realvector;
    V1, V2: StatVector;
    i: integer;
    R: extended;
begin
    InitStatVector(V1, 5);
    InitStatVector(V2, 5);
    try
    V1.Vector^[1]:= 0.8;
    V1.Vector^[2]:= 1;
    V1.Vector^[3]:= 1.2;
    V1.Vector^[4]:= 1.4;
    V1.Vector^[5]:= 1.6;

    V2.Vector^[1]:= 24;
    V2.Vector^[2]:= 20;
    V2.Vector^[3]:= 10;
    V2.Vector^[4]:= 13;
    V2.Vector^[5]:= 12;

    PolynomRegression(V, R, V1, V2, 3);
    Memo1.Lines.add('3rd degree polynom:');
    for i:= 1 to V.Dimension do
    Memo1.Lines.add('coefficient' + IntToStr(i) + ': ' + FloatToStr(V.Vector[i]));
    Memo1.Lines.add('coefficient of det. ' + FloatToStr(r));

    finally
    DestroyStatVector(V1);
    DestroyStatVector(V2);
    end;
end;

{ Fit data into a polynom of degree 5:
  a + b*x + c*x + d*x + e*x^4 + f*x^5 = y }
procedure TForm1.Higher1Click(Sender: TObject);
var V: realvector;
    V1, V2: StatVector;
    i: integer;
    R: extended;
begin
    InitStatVector(V1, 11);
    InitStatVector(V2, 11);
    try
    V1.Vector^[1]:= 10;
    V1.Vector^[2]:= 12;
    V1.Vector^[3]:= 15;
    V1.Vector^[4]:= 17;
    V1.Vector^[5]:= 20;
    V1.Vector^[6]:= 22;
    V1.Vector^[7]:= 25;
    V1.Vector^[8]:= 27;
    V1.Vector^[9]:= 30;
    V1.Vector^[10]:= 32;
    V1.Vector^[11]:= 35;

    V2.Vector^[1]:= 0.95;
    V2.Vector^[2]:= 1.05;
    V2.Vector^[3]:= 1.25;
    V2.Vector^[4]:= 1.41;
    V2.Vector^[5]:= 1.73;
    V2.Vector^[6]:= 2;
    V2.Vector^[7]:= 2.53;
    V2.Vector^[8]:= 2.98;
    V2.Vector^[9]:= 3.85;
    V2.Vector^[10]:= 4.59;
    V2.Vector^[11]:= 6.02;

    PolynomRegression(V, R, V1, V2, 5);
    for i:= 1 to V.Dimension do
    Memo1.Lines.add('coefficient' + IntToStr(i) + ': ' + FloatToStr(V.Vector[i]));
    Memo1.Lines.add('coefficient of det. ' + FloatToStr(r));

    { Draw the curve and the data points. }
    InitStatVector(SGraph.SX, 11);
    InitStatVector(SGraph.SY, 11);
    for i:= 1 to V1.Dimension do
    begin
    SGraph.SX.Vector^[i]:= V1.Vector^[i];
    SGraph.SY.Vector^[i]:= V2.Vector^[i];
    end;
    SGraph.Polyn.Dimension:= V.Dimension;
    for i:= 1 to V.Dimension do
    begin
    SGraph.Polyn.Vector[i]:= V.Vector[i];
    end;
    SGraph.DrawLinRegCurve:= false;
    StatGraphForm.ShowModal;

    finally
    DestroyStatVector(V1);
    DestroyStatVector(V2);
    end;
end;

{ Find the polynom (up to degree 10) that fits the data best.  }
procedure TForm1.BestFit2Click(Sender: TObject);
var V: realvector;
    V1, V2: StatVector;
    i: integer;
    R: extended;
begin
    InitStatVector(V1, 11);
    InitStatVector(V2, 11);
    try
    V1.Vector^[1]:= 10;
    V1.Vector^[2]:= 12;
    V1.Vector^[3]:= 15;
    V1.Vector^[4]:= 17;
    V1.Vector^[5]:= 20;
    V1.Vector^[6]:= 22;
    V1.Vector^[7]:= 25;
    V1.Vector^[8]:= 27;
    V1.Vector^[9]:= 30;
    V1.Vector^[10]:= 32;
    V1.Vector^[11]:= 35;

    V2.Vector^[1]:= 0.95;
    V2.Vector^[2]:= 1.05;
    V2.Vector^[3]:= 1.25;
    V2.Vector^[4]:= 1.41;
    V2.Vector^[5]:= 1.73;
    V2.Vector^[6]:= 2;
    V2.Vector^[7]:= 2.53;
    V2.Vector^[8]:= 2.98;
    V2.Vector^[9]:= 3.85;
    V2.Vector^[10]:= 4.59;
    V2.Vector^[11]:= 6.02;

    BestPolynomFit(V, R, V1, V2, 10);
    for i:= 1 to V.Dimension do
    Memo1.Lines.add('coefficient' + IntToStr(i) + ': ' + FloatToStr(V.Vector[i]));
    Memo1.Lines.add('coefficient of det. ' + FloatToStr(r));

    { Draw the curve and the data points. }
    InitStatVector(SGraph.SX, 11);
    InitStatVector(SGraph.SY, 11);
    for i:= 1 to V1.Dimension do
    begin
    SGraph.SX.Vector^[i]:= V1.Vector^[i];
    SGraph.SY.Vector^[i]:= V2.Vector^[i];
    end;
    SGraph.Polyn.Dimension:= V.Dimension;
    for i:= 1 to V.Dimension do
    begin
    SGraph.Polyn.Vector[i]:= V.Vector[i];
    end;
    SGraph.DrawLinRegCurve:= false;
    StatGraphForm.ShowModal;

    finally
    DestroyStatVector(V1);
    DestroyStatVector(V2);
    end;
end;

{ Calculate the coeffic. a, b, c so that the data supplied in statistical vectors X, Y, T fits
  t = a + b*x + c*y.
  This is a linear equation in two independent variables.
  The technique is called Multiple Linear Regression.                                    }
procedure TForm1.Twoindepvar1Click(Sender: TObject);
var a, b, c, r: extended;
    V1, V2, V3, V: StatVector;
    i: integer;
begin
    InitStatVector(V1, 4);
    InitStatVector(V2, 4);
    InitStatVector(V3, 4);
    InitStatVector(V, 4);
    try
    V1.Vector^[1]:= 1.5;
    V1.Vector^[2]:= 0.45;
    V1.Vector^[3]:= 1.8;
    V1.Vector^[4]:= 2.8;

    V2.Vector^[1]:= 0.7;
    V2.Vector^[2]:= 2.3;
    V2.Vector^[3]:= 1.6;
    V2.Vector^[4]:= 4.5;

    V3.Vector^[1]:= 2.1;
    V3.Vector^[2]:= 4;
    V3.Vector^[3]:= 4.1;
    V3.Vector^[4]:= 9.4;

    MultiLinearRegression1(a,b,c,r,V1,V2,V3);
    Memo1.Lines.add('a:= ' + FloatToStr(a));
    Memo1.Lines.add('b:= ' + FloatToStr(b));
    Memo1.Lines.add('c:= ' + FloatToStr(c));
    Memo1.Lines.add('r:= ' + FloatToStr(r));

    { Find the errors in the regression coefficients of the curve in every point. }
    MultiLinearRegression1Deviations(V, a, b, c, V1, V2, V3);
    Memo1.Lines.add('Deviations of coefficients:');
    for i:= 1 to V.Dimension do
    Memo1.Lines.add('Point ' + IntToStr(i) + ': ' + FloatToStr(V.Vector^[i]));

    finally
    DestroyStatVector(V1);
    DestroyStatVector(V2);
    DestroyStatVector(V3);
    DestroyStatVector(V);
    end;
end;

{ Same but now for three independent variables.
  The equation: t = a + b*x + c*y + d*z.           }
procedure TForm1.Threeindepvariables1Click(Sender: TObject);
var a, b, c, d, r: extended;
    V1, V2, V3, V4, V: StatVector;
    i: integer;
begin
    InitStatVector(V1, 5);
    InitStatVector(V2, 5);
    InitStatVector(V3, 5);
    InitStatVector(V4, 5);
    InitStatVector(V, 5);
    try

    V1.Vector^[1]:= 7;
    V1.Vector^[2]:= 1;
    V1.Vector^[3]:= 11;
    V1.Vector^[4]:= 11;
    V1.Vector^[5]:= 7;

    V2.Vector^[1]:= 25;
    V2.Vector^[2]:= 29;
    V2.Vector^[3]:= 56;
    V2.Vector^[4]:= 31;
    V2.Vector^[5]:= 52;

    V3.Vector^[1]:= 6;
    V3.Vector^[2]:= 15;
    V3.Vector^[3]:= 8;
    V3.Vector^[4]:= 8;
    V3.Vector^[5]:= 6;

    V4.Vector^[1]:= 60;
    V4.Vector^[2]:= 52;
    V4.Vector^[3]:= 20;
    V4.Vector^[4]:= 47;
    V4.Vector^[5]:= 33;

    MultiLinearRegression2(a,b,c,d,r,V1,V2,V3,V4);
    Memo1.Lines.add('a:= ' + FloatToStr(a));
    Memo1.Lines.add('b:= ' + FloatToStr(b));
    Memo1.Lines.add('c:= ' + FloatToStr(c));
    Memo1.Lines.add('d:= ' + FloatToStr(d));
    Memo1.Lines.add('r:= ' + FloatToStr(r));

    { Find the errors in the regression coefficients of the curve in every point. }
    MultiLinearRegression2Deviations(V, a, b, c, d, V1, V2, V3, V4);
    Memo1.Lines.add('Deviations of coefficients:');
    for i:= 1 to V.Dimension do
    Memo1.Lines.add('Point ' + IntToStr(i) + ': ' + FloatToStr(V.Vector^[i]));

    finally
    DestroyStatVector(V1);
    DestroyStatVector(V2);
    DestroyStatVector(V3);
    DestroyStatVector(V4);
    DestroyStatVector(V);
    end;
end;

{ Given a set of paired observations from two normal populations X and Y with unknown means 1 and 2,
  the test statistic t can be used to test the null hypothesis: 1 = 2.              }
procedure TForm1.Pairedt1Click(Sender: TObject);
var t: extended;
    V1, V2: StatVector;
begin
    InitStatVector(V1, 5);
    InitStatVector(V2, 5);
    try
    V1.Vector^[1]:= 14;
    V1.Vector^[2]:= 17.5;
    V1.Vector^[3]:= 17;
    V1.Vector^[4]:= 17.5;
    V1.Vector^[5]:= 15.4;

    V2.Vector^[1]:= 17;
    V2.Vector^[2]:= 20.7;
    V2.Vector^[3]:= 21.6;
    V2.Vector^[4]:= 20.9;
    V2.Vector^[5]:= 17.2;
    t:= PairedtStatistic(V1, V2);
    Memo1.Lines.add('t:= ' + FloatToStr(t));
    finally
    DestroyStatVector(V1);
    DestroyStatVector(V2);
    end;
end;

{ Find the value of the chi-square statistic for the goodness of fit for the following
  set of data ...                                                                       }
procedure TForm1.chi1Click(Sender: TObject);
var chi: extended;
    V1, V2: StatVector;
begin
    InitStatVector(V1, 6);
    InitStatVector(V2, 6);
    try
    V1.Vector^[1]:= 8;
    V1.Vector^[2]:= 50;
    V1.Vector^[3]:= 47;
    V1.Vector^[4]:= 56;
    V1.Vector^[5]:= 5;
    V1.Vector^[6]:= 14;

    V2.Vector^[1]:= 9.6;
    V2.Vector^[2]:= 46.75;
    V2.Vector^[3]:= 51.85;
    V2.Vector^[4]:= 54.4;
    V2.Vector^[5]:= 8.25;
    V2.Vector^[6]:= 9.15;

    chi:= ChiSquareStatistic(V1, V2);
    Memo1.Lines.add('chi:= ' + FloatToStr(chi));
    finally
    DestroyStatVector(V1);
    DestroyStatVector(V2);
    end;
end;

{ Shows information about ChroMathsome version. }
procedure TForm1.Version1Click(Sender: TObject);
begin
     ChroMathsomeVersion;
end;

procedure TForm1.Close1Click(Sender: TObject);
begin
     Close;
end;


{ Test the null hypothesis that H0: 1 = 2 (i.e. d = 0) for the data in V1 and V2. }
procedure TForm1.tStat1Click(Sender: TObject);
var t: extended;
    V1, V2: StatVector;
begin
    InitStatVector(V1, 8);
    InitStatVector(V2, 10);
    try
    V1.Vector^[1]:= 79;
    V1.Vector^[2]:= 84;
    V1.Vector^[3]:= 108;
    V1.Vector^[4]:= 114;
    V1.Vector^[5]:= 120;
    V1.Vector^[6]:= 103;
    V1.Vector^[7]:= 122;
    V1.Vector^[8]:= 120;

    V2.Vector^[1]:= 91;
    V2.Vector^[2]:= 103;
    V2.Vector^[3]:= 90;
    V2.Vector^[4]:= 113;
    V2.Vector^[5]:= 108;
    V2.Vector^[6]:= 87;
    V2.Vector^[7]:= 100;
    V2.Vector^[8]:= 80;
    V2.Vector^[9]:= 99;
    V2.Vector^[10]:= 54;

    t:= tStatistic(V1, V2, 0); { d = 0 }
    Memo1.Lines.add('t:= ' + FloatToStr(t));
    finally
    DestroyStatVector(V1);
    DestroyStatVector(V2);
    end;
end;

procedure TForm1.Example1Click(Sender: TObject);
var V: realvector;
    V1, V2: StatVector;
    i: integer;
    R: extended;
begin
    InitStatVector(V1, 5);
    InitStatVector(V2, 5);
    try
    V1.Vector^[1]:= 0.1;
    V1.Vector^[2]:= 0.2;
    V1.Vector^[3]:= 0.3;
    V1.Vector^[4]:= 0.4;
    V1.Vector^[5]:= 0.5;

    V2.Vector^[1]:= -0.055;
    V2.Vector^[2]:= 0.094;
    V2.Vector^[3]:= 0.314;
    V2.Vector^[4]:= 0.756;
    V2.Vector^[5]:= 1.138;


    PolynomRegression(V, R, V1, V2, 2);
    Memo1.Lines.add('2nd degree polynom:');
    for i:= 1 to V.Dimension do
    Memo1.Lines.add('coefficient' + IntToStr(i) + ': ' + FloatToStr(V.Vector[i]));
    Memo1.Lines.add('coefficient of det. ' + FloatToStr(r));
    Memo1.Lines.add('experimental value of g. ' + FloatToStr(2*V.Vector[V.Dimension]));
    finally
    DestroyStatVector(V1);
    DestroyStatVector(V2);
    end;
end;

end.
